local L = Capitalist.Locals

function Capitalist:UpdateRestockStack()
	self.inTradeskill = false

	self.restock = {}
	for recipe, data in pairs(self.db.profile.recipes) do
		if (data.isUsed == true and data.isFiltered == false and data.scrollCount > 0) then
			local count = data.scrollCount
			-- search AH and all inventories on that server for this scroll
			for player, t in pairs(self.db.profile.storage[GetCVar("realmName")]) do
				if (t.AH[data.scrollName]) then count = count - t.AH[data.scrollName] end
				if (t.inventory[data.scrollName]) then count = count - t.inventory[data.scrollName] end
			end
			
			if (count > 0) then
				self.restock[recipe] = count
			end
		end
	end
	
	self.requiredReagents = {}
	for recipe, count in pairs(self.restock) do
		for reagent, reagentCount in pairs(self.db.profile.recipes[recipe].reagents) do
			if (self.requiredReagents[reagent] == nil) then
				self.requiredReagents[reagent] = reagentCount * count
			else
				self.requiredReagents[reagent] = self.requiredReagents[reagent] + reagentCount * count
			end
		end
	end
end

function Capitalist:ShowRestockStack()
	self:UpdateRestockStack()

	local list = {}
	local numRestock = 0
	local reagents = {}
	for k, v in pairs(self.requiredReagents) do reagents[k] = v end
	
	for recipe, count in pairs(self.restock) do
		list[recipe] = count
		numRestock = numRestock + count
	end
	
	-- search for the required reagents in our inventory
	for bag = 0, 4 do
		for slot = 0, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (reagents[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					reagents[name] = reagents[name] - count
				end
			end
		end
	end
	
	-- print out the restock stack
	if (numRestock == 0) then
		self:Print(L["Nothing to restock!"])
		return
	else
		self:Print(string.format(L["Need to restock %d scrolls:"], numRestock))
		for item, count in pairs(list) do
			ChatFrame1:AddMessage("  "..count.."x "..item)
		end
	end
	
	-- print out any reagents missing
	list = {}
	local numMissing = 0
	for reagent, count in pairs(reagents) do
		if (count > 0) then
			list[reagent] = count
			numMissing = numMissing + count
		end
	end
	
	if (numMissing == 0) then
		self:Print(L["Everything available!"])
	else
		self:Print(L["Missing reagents:"])
		for item, count in pairs(list) do
			ChatFrame1:AddMessage("  "..count.."x "..item)
		end
	end
end

function Capitalist:Restock()
	if (self.inTradeskill) then return end

	SetTradeSkillSubClassFilter(0, 1, 1)
	SetTradeSkillInvSlotFilter(0, 1, 1)
	ExpandTradeSkillSubClass(1)
	SetTradeSkillItemNameFilter("")
	
	for k, v in pairs(self.restock) do
		if (self:TryRestock(k)) then
			return
		end
	end
	
	self:Print(L["Could not restock anything!"])
end

function Capitalist:TryRestock(name)
	local vellum
	local bag
	local slot

	--self:Print("TryRestock")
	
	for reagent in pairs(self.db.profile.recipes[name].reagents) do
		if (reagent:find(L["Enchanting Vellum"])) then
			vellum = reagent
		end
	end
	
	for i = 0, 4 do
		for j = 1, GetContainerNumSlots(i) do
			local link = GetContainerItemLink(i, j)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (name == vellum) then
					bag = i
					slot = j
				end
			end
		end
	end

--brian
--if (0 > 0) then
--	if (not bag and not vellum:find("III") and self.db.profile.useMaxVellum) then
--		for i = 0, 4 do
--			for j = 1, GetContainerNumSlots(i) do
--				local link = GetContainerItemLink(i, j)
--				if (link) then
--					local name = link:match("%[(.*)%]")
--					if (name == vellum .. " III") then
--						bag = i
--						slot = j
--					end
--				end
--			end
--		end
--	end
--brian
--end
	
	if (not bag) then
		return false
	end
	
	for i = 1, GetNumTradeSkills() do
		local skill, _, numAvailable = GetTradeSkillInfo(i)
		if (skill == name and numAvailable > 0) then
--brian
--self:Print(skill)
			DoTradeSkill(i)
			UseContainerItem(bag, slot)
			self.inTradeskill = true
			return true
		end
	end

	return false
end

function Capitalist:CHAT_MSG_LOOT(...)
	local event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...;
--	self:Print("CHAT_MSG_LOOT")
--	self:Print(arg1)	--You create: [Scroll of Enchant Chest - Powerful Stats].
--	self:Print(arg7)	--0
--	self:Print(arg8)	--0
--	self:Print(arg9)	--(empty string)
--	self:Print(arg10)	--0
--	self:Print(arg11)	--1123

	if (string.find(arg1, L["You create:"])) then
		local item = string.match(arg1, "%[(.*)%]")
		if (item and self.list[item]) then
			item = self.list[item].recipe
			self.restock[item] = self.restock[item] - 1
			if (self.restock[item] <= 0) then
				self.restock[item] = nil
			end
			self.db.profile.recipes[item].numCreated = self.db.profile.recipes[item].numCreated + 1
			self:UpdateStatistics()
		end

		self.inTradeskill = false
	end
end

function Capitalist:UNIT_SPELLCAST_INTERRUPTED(...)
	local event, arg1, arg2, arg3, arg4, arg5 = ...;
--	self:Print("UNIT_SPELLCAST_INTERRUPTED")
--	self:Print(arg1)
--	self:Print(arg2)
--	self:Print(arg4)

	if (arg1 == "player") then
		self.inTradeskill = false
	end

end

function Capitalist:UI_ERROR_MESSAGE(...)
	local event, arg1 = ...;
--	self:Print("UI_ERROR_MESSAGE")
--	self:Print(arg1)

	if (arg1 == L["Inventory is full."]) then
		self.inTradeskill = false
	end
end

function Capitalist:BAG_UPDATE(...)
	local event, arg1 = ...;
--	self:Print("BAG_UPDATE")
--	self:Print(arg1)

	-- save all scrolls and enchanting reagents in that players inventory
	local itemsInInv = self.db.profile.storage[GetCVar("realmName")][UnitName("player")].inventory
	for k, v in pairs(itemsInInv) do itemsInInv[k] = nil end
	
	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (self.list[name] or self.db.profile.reagents[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					
					if (not itemsInInv[name]) then
						itemsInInv[name] = count
					else
						itemsInInv[name] = itemsInInv[name] + count
					end
				end
			end
		end
	end

	self.PostingPostActive = false
	self.PostingSplitActive = false
--self:Print("bools")
--self:Print(self.mode)
--	if (self.mode == "posting") then
--		self:OnUpdatePosting()
--	end
end

function Capitalist:BANKFRAME_OPENED(...)
--	self:Print("BANKFRAME_OPENED")

	self.bankFrameOpen = true
end

function Capitalist:BANKFRAME_CLOSED(...)
--	self:Print("BANKFRAME_CLOSED")

	if (not self.bankFrameOpen) then return end
	self.bankFrameOpen = false

	local itemsInBank = self.db.profile.storage[GetCVar("realmName")][UnitName("player")].bank
	for k, v in pairs(itemsInBank) do itemsInBank[k] = nil end
	
	local containersToCheck = {-1, 5, 6, 7, 8, 9, 10, 11}
	
	for _, bag in pairs(containersToCheck) do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (self.list[name] or self.db.profile.reagents[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					
					if (not itemsInBank[name]) then
						itemsInBank[name] = count
					else
						itemsInBank[name] = itemsInBank[name] + count
					end
				end
			end
		end
	end
end
