local L = Capitalist.Locals

function Capitalist:StartScan()
	QueryAuctionItems(L["Scroll of Enchant"],nil,nil,0,4,6,0,0,0,nil)
	self.page = 0
	
	self.auctionUI.lastScanLabel:Hide()
	self.auctionUI.progressLabel:Show()
	self.auctionUI.progressLabel:SetText(L["scanning.."])
	self.auctionUI.progress:Show()
	self.auctionUI.progress:SetWidth(1)
	self.auctionUI.scanButton:Disable()
	self.auctionUI.postButton:Disable()
	
	for scroll, data in pairs(self.list) do
		data.ownAuctions = 0
		data.otherAuctions = {}
		
		-- calculate cost of the scroll
		data.cost = 0
		for reagent, count in pairs(self.db.profile.recipes[data.recipe].reagents) do
			data.cost = data.cost + count * self.db.profile.reagents[reagent].price * 10000
		end
		
		-- add auction house cut to the price and minProfit (thanks - why did I forget that again?)
		data.cost = self.db.profile.minProfit + data.cost / 0.95 

		-- calculate maximum price
		data.bestPrice = data.cost * (1+self.db.profile.margin/100)
		if (data.bestPrice - data.cost < self.db.profile.minMargin) then
			data.bestPrice = data.cost + self.db.profile.minMargin
		end
		if (data.bestPrice - data.cost > self.db.profile.maxMargin) then
			data.bestPrice = data.cost + self.db.profile.maxMargin
		end
		data.bestPrice = math.floor(data.bestPrice) -- kills problems with float numbers
	end
	
	self.mode = "scanning"
	self.updateHandler:Show()
end

function Capitalist:OnUpdateScanning()
	-- TODO: not sure if an auction page is always loaded before CanSendAuctionQuery() is true again
	if (CanSendAuctionQuery()) then
		self.page = self.page + 1
		self:Scan()
	end
end

function Capitalist:Scan()
	QueryAuctionItems(L["Scroll of Enchant"],nil,nil,0,4,6,self.page,0,0,nil)
	
	for i = 1, 50 do
		local name, _, _, _, _, _, _, _, buyoutPrice, _, _, owner, _  = GetAuctionItemInfo("list", i);
	
		if (name == nil) then
			self:ScanFinished()
			return
		end
	
		local v = self.list[name]
		if (v) then
			if (owner == UnitName("player")) then
				v.ownAuctions = v.ownAuctions + 1
				if (v.buyoutSelf == nil or buyoutPrice < v.buyoutSelf) then
					v.buyoutSelf = buyoutPrice
				end
			else
				table.insert(v.otherAuctions, buyoutPrice)
				if (v.buyoutOthers == nil or buyoutPrice < v.buyoutOthers) then
					v.buyoutOthers = buyoutPrice
				end
			end
		end
	end
	
	local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");

	self.auctionUI.progress:SetWidth(110 * self.page * 50 / totalAuctions)
end

function Capitalist:ScanFinished()
	self.updateHandler:Hide()

	self.auctionUI.lastScanLabel:SetText(L["Last Scan: "]..date("%H:%M"))
	self.auctionUI.lastScanLabel:Show()
	self.auctionUI.progress:Hide()
	self.auctionUI.progressLabel:Hide()
	self.auctionUI.scanButton:Enable()
	
	-- consistency check
	for i = 1, GetNumAuctionItems("owner") do
		local name, _, _, _, _, _, _, _, _, _, _, _, saleStatus = GetAuctionItemInfo("owner", i);
		if (saleStatus == 0 and self.list[name]) then
			self.list[name].ownAuctions = self.list[name].ownAuctions - 1
		end
	end
	
	for k, v in pairs(self.list) do
		if (v.ownAuctions ~= 0) then
			self.auctionUI.lastScanLabel:SetText(L["Last Scan: failed"])
			self.validScan = false
			return
		end
	end

	-- calculating optimal prices by undercutting competition
	for k, v in pairs(self.list) do
		for j = 1, #v.otherAuctions do
			if (v.otherAuctions[j] <= v.bestPrice) then
				if (v.otherAuctions[j] > v.cost) then
					v.bestPrice = v.otherAuctions[j] - 1
				end
			end
		end
	end

	self.validScan = true	
	self:UpdateAuctionUI()
	self.auctionUI.postButton:Enable()
	self:CheckForCancel()
end

function Capitalist:CheckForCancel()
	self.toCancel = {}

	for i = 1, GetNumAuctionItems("owner") do
		local name, _, _, _, _, _, _, _, buyoutPrice, bidAmount, _, _, _ = GetAuctionItemInfo("owner", i);
		if (self.list[name] and bidAmount == 0) then
			if (buyoutPrice > self.list[name].bestPrice and self.db.profile.cancelYellow) then
				table.insert(self.toCancel, i)
			elseif (buyoutPrice < self.list[name].bestPrice and self.db.profile.cancelblue) then
				table.insert(self.toCancel, i)
			elseif (self.db.profile.cancelDuration and GetAuctionItemTimeLeft("owner", i) ~= 4) then
				table.insert(self.toCancel, i)
			end
		end
	end

	if (#self.toCancel > 0) then
		self.auctionUI.cancelButton:Enable()
	else
		self.auctionUI.cancelButton:Disable()
	end
end

function Capitalist:CancelAuctions()
	for i = 1, table.getn(self.toCancel) do
		CancelAuction(self.toCancel[i])
	end
	self.toCancel = {}
	self.auctionUI.cancelButton:Disable()
end